// LIST.H -  List

#if !defined (__LIST_H)
#define  __LIST_H

#if !defined (__LINK_H)
#include "LINK.H"
#endif

#if !defined (__ITERATOR_H)
#include "ITERATOR.H"
#endif

template<class T>
class List
{
public:
   List() : first(NULL), last(NULL), count(0)  { }  //   
   ~List() { free(); }                              //  
 void insert(Iterator<T> iter, T s);   //      iter
 void push_back(T s) { insert(end(), s); }           //     
 void push_front(T s) { insert(begin(), s); }        //   
 void erase(Iterator<T> pos);                        //  
 void free();                                        //  
 Iterator<T> begin() const;                         //  
 Iterator<T> end() const;                           //  
 unsigned int size() const { return count; }        //   
 List& operator =(const List& b);                   //   
private:
 Link<T> *first;                                    //    
 Link<T> *last;                                     //    
 unsigned int count;                                //    
};

template<class T>
void List<T>::insert(Iterator<T> iter, T s)
{ //      "iter"
 Link<T> *newlink = new Link<T>(s); //    
 newlink->next = iter.position;  //      "iter"
 if (newlink->next)  //         ?
 { // 
  //        
  newlink->previous = newlink->next->previous;
  //             
  newlink->next->previous = newlink;
 }
 else
 { //        
  //         
  newlink->previous = last;
  last = newlink;          //       
 }
 if (!newlink->previous)   //         ?
 first = newlink;          //  -->       
 else                      //        
 //            
 newlink->previous->next = newlink;
 count++;                  //       1
}

template<class T>
void List<T>::erase(Iterator<T> iter)
{ //      "iter"
  assert(iter.position != NULL); //  ,  iter   
 Link<T> *remove = iter.position; //   ,    
 if (!remove->previous)        //       ?
 first = remove->next;         //  -->       
 else                          // 
 //         ,   
 //     
 remove->previous->next = remove->next;
 if (!remove->next)            //       ?
 last = remove->previous;      //  -->    
 else                          //     
 //         ,   
 //     
 remove->next->previous = remove->previous;
 iter.position = remove->next; //      
 delete remove;                //      
 count--;                      //       1
}

template<class T>
void List<T>::free()
{ //     
 Link<T> *remove, *newlink;     //      
 for (newlink = first; newlink; ) //   
 {
  remove = newlink;    //       
  newlink = newlink->next; //    
  delete remove;           //   
 }
 first = last = NULL;  //         
 count = 0;            //    
}

template<class T>
Iterator<T> List<T>::begin() const
{ //      
 Iterator<T> iter;         //  
 iter.position = first;    //      
 iter.last = last;            //       
 return iter;
}

template<class T>
Iterator<T> List<T>::end() const
{ //        
 Iterator<T> iter;         //  
 iter.position = NULL;     //        
 iter.last = last;         //       
 return iter;
}

template<class T>
List<T>& List<T>::operator =(const List<T>& b)
{ //   
 if (&b != this)   //      
 { // 
  Iterator<T> iter;          // ,      -
  free();                    //       
  //            
  // 
  for (iter = b.begin(); iter != b.end(); iter++)
  push_back(*iter);
 }
 return *this;
}

#endif  // __LIST_H
